package com.ningdatech.pmapi.workbench.manage;

import cn.hutool.core.bean.BeanUtil;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.pmapi.common.helper.UserInfoHelper;
import com.ningdatech.pmapi.projectdeclared.manage.DeclaredProjectManage;
import com.ningdatech.pmapi.projectdeclared.manage.DefaultDeclaredProjectManage;
import com.ningdatech.pmapi.projectdeclared.model.po.DeclaredProjectStatisticsPO;
import com.ningdatech.pmapi.projectlib.manage.ProjectLibManage;
import com.ningdatech.pmapi.projectlib.model.req.ProjectListReq;
import com.ningdatech.pmapi.sys.enumeration.NoticeTypeEnum;
import com.ningdatech.pmapi.sys.manage.NoticeManage;
import com.ningdatech.pmapi.sys.model.req.NoticeListReq;
import com.ningdatech.pmapi.sys.model.vo.NoticeListItemVO;
import com.ningdatech.pmapi.todocenter.manage.TodoCenterManage;
import com.ningdatech.pmapi.todocenter.model.req.ToBeProcessedReq;
import com.ningdatech.pmapi.todocenter.model.vo.ResToBeProcessedVO;
import com.ningdatech.pmapi.todocenter.model.vo.TodoCenterStatisticsVO;
import com.ningdatech.pmapi.todocenter.model.vo.TodoVO;
import com.ningdatech.pmapi.user.security.auth.model.UserFullInfoDTO;
import com.ningdatech.pmapi.user.security.auth.model.UserInfoDetails;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import com.ningdatech.pmapi.workbench.converter.WorkbenchConverter;
import com.ningdatech.pmapi.workbench.model.vo.WorkbenchVO;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.stream.Collectors;

/**
 * @Classname WorkbenchManage
 * @Description
 * @Date 2023/3/20 10:56
 * @Author PoffyZhang
 */
@Component
@AllArgsConstructor
public class WorkbenchManage {

    private final TodoCenterManage todoCenterManage;

    private final DefaultDeclaredProjectManage defaultDeclaredProjectManage;

    private final DeclaredProjectManage declaredProjectManage;

    private final NoticeManage noticeManage;

    private final UserInfoHelper userInfoHelper;

    private final ProjectLibManage projectLibManage;

    public WorkbenchVO getWorkbenchData(Integer year){
        WorkbenchVO res = new WorkbenchVO();

        //1.待办中心数据
        TodoCenterStatisticsVO statisticsVO = todoCenterManage.todoCenterStatistics();
        ToBeProcessedReq toBeProcessedReq = new ToBeProcessedReq();
        toBeProcessedReq.setPageNumber(1);
        toBeProcessedReq.setPageSize(5);
        statisticsVO.setTodoList(todoCenterManage.todoProjectList(toBeProcessedReq).getRecords()
                .stream().map(v -> WorkbenchConverter.convert(v)).collect(Collectors.toList()));
        res.setTodoCerter(statisticsVO);

        //2.项目统计数据
        res.setOrgDeclared(WorkbenchConverter.convert(defaultDeclaredProjectManage.declaredProjectOrgStatistics(year)));
        if(userInfoHelper.isSuperOrRegionAdmin(LoginUserUtil.getUserId())){
            res.setRegionDeclared(WorkbenchConverter.convert(defaultDeclaredProjectManage.declaredProjectRegionStatistics(year)));
        }
        ProjectListReq projectListReq = new ProjectListReq();
        projectListReq.setPageNumber(1);
        projectListReq.setPageSize(5);
        projectListReq.setProjectYear(year);
        res.setProjects(projectLibManage.projectLibListWithPermission(projectListReq).getRecords().stream().collect(Collectors.toList()));

        //3.所有公告按类型分
        NoticeListReq noticeListReq = new NoticeListReq();
        noticeListReq.setPageNumber(1);
        noticeListReq.setPageSize(1000);
        res.setNoticeList(noticeManage.listToMapByManager(noticeListReq));

        return res;
    }
}
